.floorplan-raw{ display: none }
.floorplan-widget{
    width: 100%;
    height: 500px;
    padding: 0;
    margin: 0;
    background: #FFF;
}
.floorplan-image img{
    width:50%;
}
p.change-image{
    margin-left: 170px;
}
